package com.example.droptik;

import android.Manifest;
import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.ImageView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.location.places.Places;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;

import com.google.maps.android.data.kml.KmlContainer;
import com.google.maps.android.data.kml.KmlLayer;
import com.google.maps.android.data.kml.KmlPlacemark;
import com.google.maps.android.data.kml.KmlPolygon;

import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.InputStream;


public class Mapa extends AppCompatActivity implements OnMapReadyCallback, GoogleApiClient.OnConnectionFailedListener {
    // definovani proměných
    private static final String TAG = "Mapa";
    private static final String FINE_LOCATION = Manifest.permission.ACCESS_FINE_LOCATION; //proměnná pro získání polohy zařízení přes WIFI a GPS
    private static final String COURSE_LOCATION = Manifest.permission.ACCESS_COARSE_LOCATION; //proměnná pro získání polohy zařízení přes GPS
    private static final int LOCATION_PERMISSION_REQUEST_CODE = 1234; //nastavení přístupového kódu k získání polohy
    private static final float DEFAULT_ZOOM = 15f; //výchozí nastavení zoomu mapy

    private ImageView mGps; //obrázkové tlačítko pro zobrazení polohy ovladače
    private ImageView mSouradniceAuta; //obrázkové tlačítko pro zobrazení polohy auta
    private ImageView mKML; //obrázkové tlačítko pro zobrazení trasy auta ze souboru GPS
    private ImageView mPolyLajna; //obrázkové tlačítko pro zobrazení trasy mezi ovladačem a autem pomocí křivky

    private Boolean ziskaneOpravneni = false; //ověření, zda jsou získána všechna oprávnění
    private GoogleMap mMap; //vrstva google map
    private FusedLocationProviderClient fusedLocationProviderClient; //klient přístupu k poloze zařízení
    private GoogleApiClient googleApiClient; //definice proměnné nutné k přístupu k externímu API

    private Bundle bundle; //přechod do nové aktivity, nebo načtení dat z přenesené z jiné aktivity
    private double zemepisnaSirkaOvladace; //zeměpisná šířka ovladače
    private double zemepisnaDelkaOvladace; //zeměpisná délka ovladače
    private double zemepisnaSirkaAuta; //zeměpisná šířka auta
    private double zemepisnaDelkaAuta; //zeměpisná délka auta

    private Nabidka nabidka = new Nabidka(); //nová instance třídy Nabidka

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {
//automaticky generovaná metoda
    }

    //metoda proběhne vždy po startu aktivity
    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState); //dědění prvků obecné aktivity
        setContentView(R.layout.activity_mapa); //přiřazení XML souboru s designem

        //definování toolbaru pro umístění menu
        Toolbar toolbar = findViewById(R.id.toolMapa);
        setSupportActionBar(toolbar);

        //přiřazení prvků z XML designu
        mGps = findViewById(R.id.ic_gps);
        mKML = findViewById(R.id.ic_KML);
        mSouradniceAuta = findViewById(R.id.ic_import);
        mPolyLajna = findViewById(R.id.ic_polyLajna);
        bundle = getIntent().getExtras();

        ziskatOpravneni(); //volání metody
    }

    //po zvolení prvku z menu přesun na metodu třídy Nabidka pro zobrazení konkrétní obrazovky
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.mePripojeni:
                nabidka.otevriPripojeniBluetooth();
                return true;
            case R.id.meOvladac:
                nabidka.otevriOvladac();
                return true;
            case R.id.meMapa:
                nabidka.otevriMapa();
                return true;
            case R.id.meZobrazeniSouradnic:
                nabidka.otevriZobrazeniSouradnicAuta();
                return true;
            case R.id.meAbout:
                nabidka.otevriAbout();
                return true;
            case R.id.meKonec:
                nabidka.ukoncitProgram();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    //tato metoda proběhne vždy a slouží ke stažení mapy z internetu
    @Override
    public void onMapReady(GoogleMap googleMap) {
        Toast.makeText(this, "Mapa je připravena", Toast.LENGTH_SHORT).show(); //hlášení že je mapa připravena
        mMap = googleMap; //přiřazení do globální proměnné (použití této konkrétní instance mapy z internetu)
        if (ziskaneOpravneni) { //jakmile jsou získána  potřebná oprávnění
            zobrazPolohuOvladace(); // volání metody pro zobrazení polohy ovladače
            if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                    != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(this,
                    Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                return;
            }
            mMap.setMyLocationEnabled(true);
            mMap.getUiSettings().setMyLocationButtonEnabled(false);

            nactiApi(); //volání metody pro načtení API
        }
    }

    //metoda která staženou mapu převede do formátu, se kterým půjde pracovat
    private void nactiApi() {
        //základní nastavení klienta pro přístup k externímu API
        googleApiClient = new GoogleApiClient
                .Builder(this)
                .addApi(Places.GEO_DATA_API)
                .addApi(Places.PLACE_DETECTION_API)
                .enableAutoManage(this, this)
                .build();

        //bloky tlačítek a volání metod po jejich stisknutí
        mGps.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                zobrazPolohuOvladace();
            }
        });

        mSouradniceAuta.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                zobrazSouradniceAuta();
            }
        });

        mKML.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                zobrazJizduZKML();
            }
        });

        mPolyLajna.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                zobrazCestuKAutu();
            }
        });

        skryjKlavesnici(); //volání metody pro skrytí klávesnice, neboť ve výchozím stavu se sama objevuje
    }

    //metoda pro grafické zobrazení polohy ovladače v mapách
    private void zobrazPolohuOvladace() {
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this); //klient, který má za úkol zjistit polohu v této aktivitě

        try {
            if (ziskaneOpravneni) {

                final Task location = fusedLocationProviderClient.getLastLocation(); //přiřaď poslední nalezenou polohu ovladače
                location.addOnCompleteListener(new OnCompleteListener() {
                    @Override
                    public void onComplete(@NonNull Task task) {
                        if (task.isSuccessful()) {
                            Location currentLocation = (Location) task.getResult(); //přiřazení polohy ovladače
                            zemepisnaSirkaOvladace = currentLocation.getLatitude(); //z přiřazené polohy ulož do globální proměnné zeměpisnou šířku ovladače
                            zemepisnaDelkaOvladace = currentLocation.getLongitude(); //z přiřazené polohy ulož do globální proměnné zeměpisnou délu ovladače

                            //vrácení mapy k poloze ovladače
                            moveCamera(new LatLng(currentLocation.getLatitude(), currentLocation.getLongitude()),
                                    DEFAULT_ZOOM,
                                    "Moje poloha");

                        } else {
                            Toast.makeText(Mapa.this, "nelze získat aktuální polohu", Toast.LENGTH_SHORT).show(); //v případě nenalezení polohu zobraz hlášení
                        }
                    }
                });
            }
        } catch (SecurityException e) {
            Log.e(TAG, "zobrazPolohuOvladace: SecurityException: " + e.getMessage());
        }
    }

    //metoda pro zobrazení polohy ovladače v mapě
    private void moveCamera(LatLng latLng, float zoom, String title) {
        mMap.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, zoom));

        if (!title.equals("Moje poloha")) {
            MarkerOptions options = new MarkerOptions()
                    .position(latLng)
                    .title(title);
            mMap.addMarker(options);
        }

        skryjKlavesnici();
    }

    //metoda, která z fragmentu v XML vytvoří vrstvu google map
    private void inicializujMapu() {
        SupportMapFragment mapFragment = (SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map);

        mapFragment.getMapAsync(Mapa.this);
    }

    //metoda pro získání potřebných oprávnění nutno ji přesně zkopírovat ze stránek Google maps api
    private void ziskatOpravneni() {
        String[] permissions = {Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION};

        if (ContextCompat.checkSelfPermission(this.getApplicationContext(),
                FINE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            if (ContextCompat.checkSelfPermission(this.getApplicationContext(),
                    COURSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
                ziskaneOpravneni = true;
                inicializujMapu();
            } else {
                ActivityCompat.requestPermissions(this,
                        permissions,
                        LOCATION_PERMISSION_REQUEST_CODE);
            }
        } else {
            ActivityCompat.requestPermissions(this,
                    permissions,
                    LOCATION_PERMISSION_REQUEST_CODE);
        }
    }

    //metoda pro zobrazení současné polohy auta
    private void zobrazSouradniceAuta() {
        String souradniceAuta = bundle.getString("souradniceAuta"); //přijetí souřadnic z třídy přenos souřadnic auta
        String[] tokens = souradniceAuta.split(","); //rozdělení souřadnic na zeměpisnou šířku a délku auta
        zemepisnaSirkaAuta = Double.parseDouble(tokens[0]); //převod stringu na číslo
        zemepisnaDelkaAuta = Double.parseDouble(tokens[1]); //převod stringu na číslo
        LatLng auto = new LatLng(zemepisnaSirkaAuta, zemepisnaDelkaAuta); //zobrazení souřadince auta na mapě
        mMap.addMarker(new MarkerOptions().position(auto).title("Pozice auta")); //po stisknutí na ukazatel zobrazení hlášky
        mMap.moveCamera(CameraUpdateFactory.newLatLng(auto)); //přesun mapy na pozici auta
    }

    //metoda pro vytvoření KML vrstvy
    private void zobrazJizduZKML() {
        InputStream kmlInputStream = getResources().openRawResource(R.raw.souradnice); //cesta k souboru, v projektu je složka raw a v ní soubor souradnice.kml
        try {
            KmlLayer kmlLayer = new KmlLayer(mMap, kmlInputStream, getApplicationContext()); //vytvoření vrstvy KML pro zobrazení souřadnic ze souboru
            kmlLayer.addLayerToMap();
            zobrazKMLNaMape(kmlLayer); //volání metody která bude číst ze souboru a zobrazí cestu auta
        } catch (IOException e) {
            e.printStackTrace();
        } catch (XmlPullParserException e) {
            e.printStackTrace();
        }
    }

    //metoda pro čtení z KML souboru a zobrazení cesty auta
    private void zobrazKMLNaMape(KmlLayer kmlLayer) {
        KmlContainer container = kmlLayer.getContainers().iterator().next(); //vrátí první kontejner v KML vrstvě
        container = container.getContainers().iterator().next(); //vrátí další kontejner po prvním
        KmlPlacemark placemark = container.getPlacemarks().iterator().next(); //vrátí první označení místa
        KmlPolygon polygon = (KmlPolygon) placemark.getGeometry(); //vytvoří polygon z míst
        LatLngBounds.Builder builder = new LatLngBounds.Builder(); //cesta ze značek
        for (LatLng latLng : polygon.getOuterBoundaryCoordinates()) { //postuponé vykreslení jednotlivých souřadnic
            builder.include(latLng);
        }
        int width = getResources().getDisplayMetrics().widthPixels;
        int height = getResources().getDisplayMetrics().heightPixels;
        mMap.moveCamera(CameraUpdateFactory.newLatLngBounds(builder.build(), width, height, 1)); // zobrazení křivky ze značek poloh
    }

    //metoda zobrazující křivku od ovladače k autu
    private void zobrazCestuKAutu(){
        mMap.addPolyline(new PolylineOptions()
         //zobrazení aktuální polohy ovladače přes metody get, nejdříve dochází k definování počátečního bodu, pak koncového a mezi tyto body votvoří čáru
        .add(new LatLng(zemepisnaSirkaOvladace, zemepisnaDelkaOvladace),new LatLng(zemepisnaSirkaAuta, zemepisnaDelkaAuta))
        .width(8f) //šířka čáry znázorňující cestu
        .color(Color.BLUE) //barva čáry
        );
    }

    //výchozí metoda API pro získání přístupu k polohám a internetu
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        ziskaneOpravneni = false;

        switch (requestCode) {
            case LOCATION_PERMISSION_REQUEST_CODE: {
                if (grantResults.length > 0) {
                    for (int i = 0; i < grantResults.length; i++) {
                        if (grantResults[i] != PackageManager.PERMISSION_GRANTED) {
                            ziskaneOpravneni = false;
                            return;
                        }
                    }
                    ziskaneOpravneni = true;
                    inicializujMapu();
                }
            }
        }
    }

    //metoda, která ukryje klávesnici
    private void skryjKlavesnici() {
        View view = this.getCurrentFocus();
        if (view != null) {
            InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
            imm.hideSoftInputFromWindow(view.getWindowToken(), 0);
        }
    }

}

