package com.example.droptik;


import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.graphics.Color;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Toast;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Set;
import java.util.UUID;

public class PrenosSouradnicAuta extends AppCompatActivity {
    private BluetoothDevice bluetoothDevice; //proměnná pro Bluetooth modul mobilního zařízení
    private BluetoothSocket bluetoothSocket = null; //proměnná pro přenos informací po Bluetooth
    private final String ADRESA_ZARIZENI = "33:33:0C:05:8F:2C"; //MAC adresa Bluetooth modulu auta
    private static final UUID uuid = UUID.fromString("00001101-0000-1000-8000-00805f9b34fb");
    private Thread thread;
    private boolean zastavenyThread; //jestli je zastaven přenos
    private InputStream inputStream; //přenos od auta k ovladači
    private OutputStream outputStream; //přenos od ovladače k autu
    private byte bafr[]; //definování velikosti vyrovnávacího bafru
    private byte surovaData[]; //pole souřadnic
    private String souradnice; //proměnná pro uchování souřadnic pro předání k mapám
    private PripojeniBluetooth pripojeniBluetooth = new PripojeniBluetooth(); //instance pro připojení Bluetooth
    private ZobrazeniSouradnicAuta zobrazeniSouradnicAuta = new ZobrazeniSouradnicAuta();

    //metoda která zahájí přenos souřadnic auta
    protected void zahajPrenos() {
        if (nactiBT() == true) { //jakmile došlo k načtení Bluetooth modulu
            if (pripojBT() == true) { //jakmile došlo k připojení k Bluetooth nastav potřebná tlačítka (nastavení viditelnosti převážně)
                zobrazeniSouradnicAuta.buttonStart.setVisibility(View.INVISIBLE);
                zobrazeniSouradnicAuta.buttonStop.setVisibility(View.VISIBLE);
                zobrazeniSouradnicAuta.buttonVymaz.setVisibility(View.VISIBLE);
                zobrazeniSouradnicAuta.buttonSouradniceMapa.setVisibility(View.VISIBLE);
                zobrazeniSouradnicAuta.buttonUloz.setVisibility(View.VISIBLE);
                zobrazeniSouradnicAuta.jeBTPripojen = true;
                zobrazeniSouradnicAuta.textViewStav.setText("Připojeno");
                zobrazeniSouradnicAuta.textViewStav.setTextColor(Color.parseColor("#20d82f"));
                try {
                    bluetoothSocket.getOutputStream().write(8); //odeslání příkazu pro zahájení přenosu
                } catch (IOException e) {
                    e.printStackTrace();
                }
                vypisSouradnice();
            }
        }
    }

    //metoda pro zobrazení přijatých souřadnic
    protected void vypisSouradnice() {
        final Handler handler = new Handler(); //handler slouží k příjmu zpráv ze vzdáleného zařízení a rovná si je do fronty
        zastavenyThread = false; //přenos ještě běží
        bafr = new byte[1024]; //snaha nastavit vyrovnávací bafr na max
        Thread thread = new Thread(new Runnable() {
            public void run() { //vytvoření vlákna pro příjem dat, provedené operace se vrací do handleru
                //cyklus vezme přijatou informaci, postupně bude docházet ke zvětšování vyrovnávacího pole a nová informace se bude vkládat do pole surovaData
                //ke konci dojde k převedení informace na string jako souřadnice, neboť mezi autem a ovladačem dochází přenosu bytů a ty se později převádí do srozumitelného formátu
                //cyklus bude trvat dokud budou přijaty nějaké souřadnice
                while (!Thread.currentThread().isInterrupted() && !zastavenyThread) { //pokud nedojde k zastavení přenosu
                    try {
                        int velikostBafru = inputStream.available();
                        if (velikostBafru > 0) {
                            surovaData = new byte[velikostBafru]; //nastavení pole souřadnic na velikost přijatých souřadnic
                            inputStream.read(surovaData); //
                            final String string = new String(surovaData, "UTF-8"); //převedení prvku pole do stringu, zde se jedná už o souřadnici
                            handler.post(new Runnable() {
                                public void run() {
                                    souradnice = string;
                                    zobrazeniSouradnicAuta.textViewSouradnice.append(string); //vypsání souřadnice do okna na poslední místo
                                }
                            });
                        }
                    } catch (IOException ex) {
                        zastavenyThread = true;
                    }
                }
            }
        });
        thread.start();
    }

    //metoda pro odeslání příkazu pro ukládání souřadnic na sd kartu
    protected void ukladejSouradniceNaSD() {
        //změna viditelnosti tlačítek
        zobrazeniSouradnicAuta.buttonUloz.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonZastavUkladani.setVisibility(View.VISIBLE);
        try {
            bluetoothSocket.getOutputStream().write(9); //odeslání příkazu
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //metoda pro zastavení ukládání na SD kartu
    protected void zastavUkladaniNaSD(){
        zobrazeniSouradnicAuta.buttonZastavUkladani.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonUloz.setVisibility(View.VISIBLE);
        try {
            bluetoothSocket.getOutputStream().write(10);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    //metoda zastaví přenos souřadnic a nastaví požadovaná tlačítka (upraví viditelnost)
    protected void ukonciPrenos() throws IOException {
        zastavenyThread = true;
        outputStream.close();
        inputStream.close();
        bluetoothSocket.close();
        zobrazeniSouradnicAuta.buttonStart.setVisibility(View.VISIBLE);
        zobrazeniSouradnicAuta.buttonStop.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonVymaz.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonSouradniceMapa.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonUloz.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.buttonUloz.setVisibility(View.INVISIBLE);
        zobrazeniSouradnicAuta.jeBTPripojen = false;
        zobrazeniSouradnicAuta.textViewStav.setText("Nepřipojeno");
        zobrazeniSouradnicAuta.textViewStav.setTextColor(Color.parseColor("#d81e2a"));
    }

    //metoda která uloží poslední souřadnici a přepne na obrazovku mapa
    protected void prejdiNaMapu() {
        if (souradnice != null) {
            Intent intentSouradnice = new Intent(this, Mapa.class); //příprava na přechod do obrazovky mapa
            intentSouradnice.putExtra("souradniceAuta", souradnice); //předání získaných souřadnic do aktivity mapa
            startActivity(intentSouradnice); //otevření samotné obrazovky
        }
    }

    protected boolean nactiBT() {
        boolean nalezeno = false;
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if (bluetoothAdapter == null) {
            Toast.makeText(getApplicationContext(), "Zařízení nepodporuje Bluetooth", Toast.LENGTH_SHORT).show();
        }
        if (!bluetoothAdapter.isEnabled()) {
            Intent zapniBluetooth = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(zapniBluetooth, 0);
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        Set<BluetoothDevice> sparovanaZarizeni = bluetoothAdapter.getBondedDevices();
        if (sparovanaZarizeni.isEmpty()) {
            Toast.makeText(getApplicationContext(), "Stisknětě znova zahajPrenos", Toast.LENGTH_SHORT).show();
        } else {
            for (BluetoothDevice i : sparovanaZarizeni) {
                if (i.getAddress().equals(ADRESA_ZARIZENI)) {
                    bluetoothDevice = i;
                    nalezeno = true;
                    break;
                }
            }
        }
        return nalezeno;
    }

    protected boolean pripojBT() {
        boolean pripojeno = true;
        try {
            bluetoothSocket = bluetoothDevice.createRfcommSocketToServiceRecord(uuid);
            bluetoothSocket.connect();
        } catch (IOException e) {
            e.printStackTrace();
            pripojeno = false;
        }
        if (pripojeno == true) {
            try {
                zobrazeniSouradnicAuta.textViewStav.setText("Připojování");
                outputStream = bluetoothSocket.getOutputStream();
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                inputStream = bluetoothSocket.getInputStream();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return pripojeno;
    }

}
