
//Pro auto bylo použito Arduino MEGA
//Bluetooth se musí nejdříve připojit přes správu androidu zařízení s odpovídající MAC adresou Bluetooth modulu, heslo pro spárování je 1234, pak teprve připojit aplikaci
#include <NeoSWSerial.h>  //knihovna pro komunikaci po sériové lince 
#include <Servo.h> // servo knihovna
#include <TinyGPS++.h> //knihovna pro GPS modul
#include <SPI.h> //knihovna pro SPI sběrnici
#include <SD.h> //knihovna pro SD modul
//definovaní globálních proměnných
Servo servoNaprava; // servo pro zadní nápravu udávající směr

/*
  připojení motoru do motorShiledu v pořadí:
  směr pravého motoru, směr levého motoru, rychlost pravého motoru, rychlost levého motoru, měřič proudu analog A0, A1, vpřed, vzad
*/
int smerPrava = 12;
int smerLeva = 13;
int rychlostPrava = 3;
int rychlostLeva = 11;
int proudPrava = A0;
int proudLeva = A1;

TinyGPSPlus gps; //TinyGPS++ objekt
double zemSirka, zemDelka;
String souradnice;

int bluetoothTx = 48; // bluetooth TX do pinu 48
int bluetoothRx = 49; // bluetooth RX do pinu 49
NeoSWSerial bluetooth(bluetoothTx, bluetoothRx);
int hodnotaBluetooth;

int servoNaprava_pozice = 90; //výchozí natočení servo motoru, 90 stupňů

File soubor; //definování souboru
int pinSS = 53; //SS pin SD modulu
int pinCS = 47; //CS pin SD modulu

void setup() {
  servoNaprava.attach(46); // připojení serva pro nápravu na port 46
  servoNaprava.write(servoNaprava_pozice); // výchozí stav otočení servomotoru nápravy

  Serial.begin(9600);
  Serial3.begin(9600); //15(GPSTX), 14(GPSRX) Arduino Mega porty použity pro GPS
  bluetooth.begin(9600);

  //nastavení výstupu motorů
  pinMode(smerPrava, OUTPUT);
  pinMode(smerLeva, OUTPUT);

  //nastavení výstupu SD modulu
  pinMode(pinSS, OUTPUT); //tento pin se nebude používat, ale musí být definován
} //konec metody setup

void loop() {
  // Pokud z GPS přicházejí nějaká data, pošli je do knihovny TinyGPS++
  while (Serial3.available() > 0) {
    if (gps.encode(Serial3.read())) {
      zemSirka = gps.location.lat(); //získání zeměpisné šířky pomocí metody knihovny TinyGPS++
      zemDelka = gps.location.lng(); //získání zeměpisné délky pomocí metody knihovny TinyGPS++
      souradnice = (String(zemSirka, 6) + ',' + String(zemDelka, 6)); //skládání souřadnice auta do požadovaného formátu na 6 desetinných míst
      delay(1000);
    }
  } //konec GPS ověření

  //Přečti hodnotu z bluetooth
  if (bluetooth.available() > 0 ) { // pokud přišla hodnota k bluetooth
    hodnotaBluetooth = bluetooth.read(); //čtení hodnoty bluetooth
    Serial.println(hodnotaBluetooth);
  } //konec bluetooth ověření

  //Na základě přijatého příkazu proveď požadovanou operaci
  switch (hodnotaBluetooth) {
    case 1: //jakmile dojde k načtení příkazu pro pomalé rozjíždění
      pomaleRozjizdeni(); //volaní metody pro pomalé rozjíždění
      break;
    case 2: //jakmile dojde k načtení příkazu pomalé couvání
      pomaleCouvani(); //volaní metody pro pomalé couvání
      break;
    case 3: //jakmile dojde k načtení příkazu pro couvání zapni oba motory na plný výkon
      dozadu(); //volaní metody vzad
      break;
    case 4:  //jakmile dojde k načtení příkazu pro jízdu vpřed plnou rychlostí
      dopredu(); //volaní metody vpřed
      break;
    case 5: //jakmile dojde k načtení příkazu stop
      zastav(); //volaní metody zastav
      break;
    case 6: //jakmile doje k načtení příkazu pro otočení doprava
      doprava(); //volaní metody doprava
      break;
    case 7: //jakmile doje k načtení příkazu pro otočení doleva
      doleva(); //volaní metody doleva
      break;
    case 8: //jakmile dojde k načtení příkazu odeslání souřadnic
      vypisSouradnic(); //volaní metody vypisSouradnic
      break;
    case 9: //jakmile dojde k načtení příkazu k ukládání souřadnic na SD kartu
      if (SD.begin(pinCS)) { //pokud se načetl SD modul
        zapisNaSD();
      }
      break;
    case 10: //jakmile dojde k načtení příkazu k ukončení ukládání souřadnic na SD kartu
      if (SD.begin(pinCS)) { //pokud se načetl SD modul
        ukonciZapisNaSD();
      }
      break;
  } //konec přepínače

  //jakmile došel příkaz z aplikace o otočení o určitý úhel dochází k zavolání těchto metod
  if (hodnotaBluetooth >= 100 && hodnotaBluetooth <= 120) { //první znak znázorňuje směr otáčení, 1 doleva zbylé dva znaky znázorňují úhel
    zabocAnalogoveVlevo();
  }
  if (hodnotaBluetooth >= 200 && hodnotaBluetooth <= 220) { //první znak znázorňuje směr otáčení, 2 doprava zbylé dva znaky znázorňují úhel
    zabocAnalogovePravo();
  }
} //konec metody loop

void pomaleRozjizdeni() {
  digitalWrite(smerPrava, HIGH); //zapni motor pro jízdu vpřed, HIGH
  digitalWrite(smerLeva, HIGH);
  analogWrite(rychlostPrava, 190); //nastvavení rychlosti skoro na plný výkon
  analogWrite(rychlostLeva, 190);
  delay(300);
}// konec metody pomaleRozjizdeni

void pomaleCouvani() {
  digitalWrite(smerPrava, LOW); //zapni motor pro jízdu vzad, LOW
  digitalWrite(smerLeva, LOW);
  analogWrite(rychlostPrava, 190); 
  analogWrite(rychlostLeva, 190);
  delay(300);
}// konec metody pomaleCouvani

void dopredu() {
  digitalWrite(smerPrava, HIGH); //zapni motor pro jízdu vpřed, HIGH
  digitalWrite(smerLeva, HIGH);
  analogWrite(rychlostPrava, 255); //nastvavení rychlosti na max
  analogWrite(rychlostLeva, 255);
  delay(300);
}// konec metody dopredu

void dozadu() {
  digitalWrite(smerPrava, LOW);//zapni motor pro jízdu vzad, LOW
  digitalWrite(smerLeva, LOW);
  analogWrite(rychlostPrava, 255); //nastvavení rychlosti couvání na max
  analogWrite(rychlostLeva, 255);
  delay(300);
}// konec metody dozadu

void zastav() {
  digitalWrite(smerPrava, LOW);//zastav motor
  digitalWrite(smerLeva, LOW);
  analogWrite(rychlostPrava, 0); //vynuluj rychlost
  analogWrite(rychlostLeva, 0);
  delay(300);
}// konec metody zastav

void doprava() {
  if (servoNaprava_pozice <= 110) { //otočit servomotor lze jen od 90 do 110 stupňů, 110 stupňů je zvoleno z důvodu konstrukce auta, která nenabízí větší otočení nápravy doprava
    servoNaprava_pozice += 1; //přičtení 1 stupně k servomotoru
    servoNaprava.write(servoNaprava_pozice);
    delay(5);
  }
}// konec metody doprava

void doleva() {
  if (servoNaprava_pozice >= 70) { //otočit servomotor lze jen od 90 do 70 stupňů, 70 stupňů je zvoleno z důvodu konstrukce auta, která nenabízí větší otočení nápravy doleva
    servoNaprava_pozice -= 1; //odečtení 1 stupně k servomotoru
    servoNaprava.write(servoNaprava_pozice);
    delay(5);
  }
}// konec metody doleva

void vypisSouradnic() {
  bluetooth.println(souradnice);
  delay(60000); //odesílání souřadnic po minutě
} // konec metody vypisSouradnic

void zapisNaSD() {
  soubor = SD.open("souradnice.csv", FILE_WRITE); //otevření souboru pro zápis
  if (soubor) { //pokud se povedlo otevřít soubor
    soubor.print(souradnice); //zápis souřadnic
    soubor.print("\n"); //odřádkování, musí zde být tento znak
  }
}// konec metody zapisNaSD

void ukonciZapisNaSD() {
  soubor.close(); //uzavření souboru
}// konec metody ukonciZapisNaSD

void zabocAnalogoveVlevo() {
  int pomocna = hodnotaBluetooth - 100; //z poslané hodnoty se získá o kolik stupňů se má natočit náprava
  servoNaprava_pozice = servoNaprava_pozice - pomocna; //natočení nápravy o daný úhel
  if (servoNaprava_pozice > 70) { //aby nedošlo k otočení nápravy přes povolenou hodnotu
    servoNaprava.write(servoNaprava_pozice); //na jakém úhlu se má náprava zastavit
    delay(5);
  }
} //konec metody zabocAnalogoveVlevo

void zabocAnalogovePravo() {
  int pomocna = hodnotaBluetooth - 200; //z poslané hodnoty se získá o kolik stupňů se má natočit náprava
  servoNaprava_pozice = servoNaprava_pozice + pomocna;
  if (servoNaprava_pozice < 110) { //aby nedošlo k otočení nápravy přes povolenou hodnotu
    servoNaprava.write(servoNaprava_pozice); //na jakém úhlu  se má náprava zastavit
    delay(5);
  }
} //konec metody zabocAnalogovePravo
